# -*- coding: utf-8 -*-
"""
Created on Sat Oct 24 13:16:09 2020

@author: kourf
"""
import sqlite3




data_types = {
                # name.basics
                "nconst": "int",  
                "primaryName": "varchar(105)",
                "birthYear": "smallint",
                "deathYear": "smallint",
                "primaryProfession": "varchar(66)",
                "knownForTitles": "int",
                # title.basics
                "tconst": "int",
                "titleType": "varchar(12)",
                "primaryTitle": "varchar(419)",
                "originalTitle": "varchar(419)",
                "isAdult": "bit",
                "startYear": "smallint",
                "endYear": "smallint",
                "runtimeMinutes": "smallint",
                "genres": "varchar(32)",
                # titre.crew
                "directors": "int",
                "writers": "int",
                # title.episodes
                "parentTconst": "int",
                "seasonNumber": "smallint",
                "episodeNumber": "smallint",
                # title.principals
                "ordering": "smallint",
                "category": "varchar(20)",
                "job": "varchar(286)",
                "characters": "varchar(463)",
                # title.ratings
                "averageRating": "float(24)",
                "numVotes": "int",
                # title.akas
                "titleId": "int",
                "title": "varchar(831)",
                "region": "varchar(4)",
                "language": "varchar(3)",
                "types": "varchar(20)",
                "attributes": "varchar(62)",
                "isOriginalTitle": "bit",
            }

file_names = [
                # "name.basics.tsv", # traitée séparément
                "title.akas.tsv", 
                "title.basics.tsv", 
                # "title.crew.tsv",  # traitée séparément
                "title.episode.tsv",
                "title.principals.tsv",
                "title.ratings.tsv"
             ]



def check_max_length_field(file_name):
    print("Loading file {}".format(file_name))
    with open(file_name, 'r', encoding="utf8") as tsv_file:
        tsv = []
        tsv = tsv_file.readlines()
    if not tsv:
        print("La lecture du fichier a échoué.")
        return
    print("Processing {} lines".format(len(tsv)))
    name = file_name[:-4].replace(".", "_")
    attributes = tsv[0].replace("\n", "").split("\t")
    nb_attributes = len(attributes)
    for i in range(len(tsv)):
        tsv[i] = tsv[i].replace("\n", "").split("\t")
    print("fin de la transformation en liste")
    for j in range(nb_attributes):    
        max_length = 0
        for i in range(1, len(tsv)):
            if len(tsv[i][j]) > max_length:
                max_length = len(tsv[i][j])
                imax = i
        print("Longueur maxi du champ {} : {} atteint ligne {} avec {}".format(attributes[j], max_length, imax, tsv[imax][j]))
 
   
# for f in file_names: check_max_length_field(f)




def create_db_from_tsv_table(file_name, db_name='imdb.db'):
    # Chargement du fichier
    print("Loading file {}".format(file_name))
    with open(file_name, 'r', encoding="utf8") as tsv_file:
        tsv = []
        tsv = tsv_file.readlines()
    if not tsv:
        print("La lecture du fichier a échoué.")
        return
    
    # Conversion des données
    attributes = tsv[0][:-1].split("\t")
    nb_attributes = len(attributes)
    print("Converting {} lines into Python lists".format(len(tsv)))
    for i in range(1, len(tsv)):
        tsv[i] = tsv[i][:-1].split("\t")
        for j, att in enumerate(attributes):
            tsvij = tsv[i][j]
            if tsvij == '\\N':
                tsv[i][j] = None
            elif data_types[att][0] != 'v':
                if att == "averageRating":
                    tsv[i][j] = float(tsvij)
                elif att in ("nconst", "tconst", "parentTconst", "titleId"):
                    tsv[i][j] = int(tsvij[2:])
                else:
                    tsv[i][j] = int(tsvij)
        
    # Création de la table
    print("Building database")
    name = file_name[:-4].replace(".", "_")
    sql_script = "CREATE TABLE {} (\n".format(name)
    if name == 'title_akas' or name == 'title_principals': 
        sql_script += "{} {}".format(attributes[0], data_types[attributes[0]])
    else:
        sql_script += "{} {} PRIMARY KEY".format(attributes[0], data_types[attributes[0]])
    for att in attributes[1:]:
        sql_script += ",\n{} {}".format(att, data_types[att])
    sql_script += "\n);\n\n"
    
    conn = sqlite3.connect('imdb.db')
    cursor = conn.cursor()
    cursor.execute("DROP TABLE IF EXISTS {}".format(name))
    cursor.execute(sql_script)
    conn.commit()
    
    # Remplissage de la table
    sql_insert = '''INSERT INTO {} VALUES (?'''.format(name) + ''', ?''' * (nb_attributes-1) + ''')'''
    cursor.executemany(sql_insert, tsv[1:])
    conn.commit()
    cursor.close()
    conn.close()






def create_name_basics_db(file_name='name.basics.tsv', db_name='imdb.db'):
    """
    Version modifiée de la fonction create_db_from_tsv_table pour normaliser la 
    table issue du fichier name.basics.tsv.
    La version json contient une colonne knownForTitles avec plusieurs valeurs
    (identifiants de films) par cellules. 
    On supprime la dernière colonne (knownForTitles) et on crée une nouvelle table 
    nommée name_titles avec les attributs nconst et knownForTitles
    """
    # Chargement du fichier
    print("Loading file {}".format(file_name))
    with open(file_name, 'r', encoding="utf8") as tsv_file:
        tsv = []
        tsv = tsv_file.readlines()
    if not tsv:
        print("La lecture du fichier a échoué.")
        return
    
    # Conversion des données
    attributes = tsv[0][:-1].split("\t")
    nb_attributes = len(attributes)-1
    print("Converting {} lines into Python lists".format(len(tsv)))
    name_titles = [] 
    for i in range(1, len(tsv)):
        entry = tsv[i][:-1].split("\t")
        entry[0] = int(entry[0][2:])
        for j in (1, 4, 5):
            if entry[j] == '\\N':
                entry[j] = None
        for j in (2,3):
            if entry[j] == '\\N':
                entry[j] = None
            else:
                entry[j] = int(entry[j])
        tsv[i] = entry[:-1]
        if entry[-1] == None:
            name_titles.append((entry[0], None))
        else:
            titles = entry[-1].split(',')
            for title in titles:
                name_titles.append((entry[0], int(title[2:])))
            
        
    # Création de la table name_basics
    print("Building database name_basics")
    name = file_name[:-4].replace(".", "_")
    sql_script = "CREATE TABLE {} (\n".format(name)
    sql_script += "{} {}".format(attributes[0], data_types[attributes[0]])
    for att in attributes[1:-1]:
        sql_script += ",\n{} {}".format(att, data_types[att])
    sql_script += "\n);\n\n"
    
    conn = sqlite3.connect(db_name)
    cursor = conn.cursor()
    cursor.execute("DROP TABLE IF EXISTS {}".format(name))
    cursor.execute(sql_script)
    conn.commit()
    
    # Remplissage de la table
    sql_insert = '''INSERT INTO {} VALUES (?'''.format(name) + ''', ?''' * (nb_attributes-1) + ''')'''
    cursor.executemany(sql_insert, tsv[1:])
    conn.commit()
    
    # Création de la table name_titles
    print("Building database name_titles")
    name = 'name_titles'
    attributes = ['nconst', 'knownForTitles']
    nb_attributes = 2
    sql_script = "CREATE TABLE {} (\n".format(name)
    sql_script += "{} {}".format(attributes[0], data_types[attributes[0]])
    for att in attributes[1:]:
        sql_script += ",\n{} {}".format(att, data_types[att])
    sql_script += "\n);\n\n"
    
    cursor.execute("DROP TABLE IF EXISTS {}".format(name))
    cursor.execute(sql_script)
    conn.commit()
    
    # Remplissage de la table
    sql_insert = '''INSERT INTO {} VALUES (?'''.format(name) + ''', ?''' * (nb_attributes-1) + ''')'''
    cursor.executemany(sql_insert, name_titles)
    conn.commit()
   
    cursor.close()
    conn.close()



def create_title_crew_db(file_name='title.crew.tsv', db_name='imdb.db'):
    """
    Version modifiée de la fonction create_db_from_tsv_table pour normaliser la 
    table issue du fichier title.crew.tsv.
    On crée deux tables, title_directors et title_writers.
    On supprime la dernière colonne (knownForTitles) et on crée une nouvelle table 
    nommée name_titles avec les attributs nconst et knownForTitles
    """
    # Chargement du fichier
    print("Loading file {}".format(file_name))
    with open(file_name, 'r', encoding="utf8") as tsv_file:
        tsv = []
        tsv = tsv_file.readlines()
    if not tsv:
        print("La lecture du fichier a échoué.")
        return
    
    # Conversion des données
    print("Converting {} lines into Python lists".format(len(tsv)))
    title_directors = []
    title_writers = []
    for entry in tsv[1:]:
        entry = entry[:-1].split("\t")
        tconst = int(entry[0][2:])
        if entry[1] == '\\N':
            title_directors.append((tconst, None))
        else:
            directors = entry[1].split(',')
            for director in directors:
                title_directors.append((tconst, int(director[2:])))
        if entry[2] == '\\N':
            title_writers.append((tconst, None))
        else:
            writers = entry[2].split(',')
            for writer in writers:
                title_writers.append((tconst, int(writer[2:])))
        
    # Création de la table title_directors
    print("Building database title_directors")
    name = 'title_directors'
    attributes = ['tconst', 'directors']
    nb_attributes = 2
    sql_script = "CREATE TABLE {} (\n".format(name)
    sql_script += "{} {}".format(attributes[0], data_types[attributes[0]])
    for att in attributes[1:]:
        sql_script += ",\n{} {}".format(att, data_types[att])
    sql_script += "\n);\n\n"
    
    conn = sqlite3.connect(db_name)
    cursor = conn.cursor()
    cursor.execute("DROP TABLE IF EXISTS {}".format(name))
    cursor.execute(sql_script)
    conn.commit()
    
    # Remplissage de la table
    sql_insert = '''INSERT INTO {} VALUES (?'''.format(name) + ''', ?''' * (nb_attributes-1) + ''')'''
    cursor.executemany(sql_insert, title_directors)
    conn.commit()
   
    # Création de la table title_writers
    print("Building database title_writers")
    name = 'title_writers'
    attributes = ['tconst', 'writers']
    nb_attributes = 2
    sql_script = "CREATE TABLE {} (\n".format(name)
    sql_script += "{} {}".format(attributes[0], data_types[attributes[0]])
    for att in attributes[1:]:
        sql_script += ",\n{} {}".format(att, data_types[att])
    sql_script += "\n);\n\n"

    cursor.execute("DROP TABLE IF EXISTS {}".format(name))
    cursor.execute(sql_script)
    conn.commit()
    
    # Remplissage de la table
    sql_insert = '''INSERT INTO {} VALUES (?'''.format(name) + ''', ?''' * (nb_attributes-1) + ''')'''
    cursor.executemany(sql_insert, title_writers)
    conn.commit()
    
    cursor.close()
    conn.close()       



def create_db():
    for f in file_names:
        create_db_from_tsv_table(f)
    create_name_basics_db()
    create_title_crew_db()


def reduce_db(db_name='imdb.db'):
    conn = sqlite3.connect(db_name)
    cursor = conn.cursor()
    sql_script = """
    DELETE FROM title_ratings WHERE numVotes < 500 OR (numVotes < 5000 AND averageRating < 6.5) ;
    DELETE FROM title_episode WHERE tconst NOT IN (SELECT tconst FROM title_ratings) 
            OR parentTconst NOT IN (SELECT tconst FROM title_ratings) ;
    DELETE FROM title_akas WHERE titleId NOT IN (SELECT tconst FROM title_ratings) ;
    DELETE FROM title_basics WHERE tconst NOT IN (SELECT tconst FROM title_ratings) ;
    DELETE FROM title_directors WHERE tconst NOT IN (SELECT tconst FROM title_ratings) ;
    DELETE FROM title_writers WHERE tconst NOT IN (SELECT tconst FROM title_ratings) ;
    DELETE FROM title_principals WHERE tconst NOT IN (SELECT tconst FROM title_ratings) ;
    DELETE FROM name_titles WHERE knownForTitles NOT IN (SELECT tconst FROM title_ratings) 
                                OR knownForTitles IS NULL ;
    
    DELETE FROM name_basics 
    WHERE nconst NOT IN 
         (SELECT nconst FROM name_basics WHERE nconst IN (SELECT directors FROM title_directors))  
      AND nconst NOT IN 
         (SELECT nconst FROM name_basics WHERE nconst IN (SELECT writers FROM title_writers)) 
      AND nconst NOT IN 
         (SELECT nconst FROM name_titles)
      AND nconst NOT IN 
         (SELECT nconst FROM title_principals) ;
    
    VACUUM ;

    """
    cursor.executescript(sql_script)
    conn.commit()
    cursor.close()
    conn.close()   

if __name__ == '__main__':
    create_db()
    reduce_db()

            